@@ -18,6 +18,8 @@ from ..uvfits import UVFITS
 
 pytest.importorskip("casacore")
 
+allowed_failures = "filename"
+
 
 @pytest.fixture(scope="session")
 def nrao_uv_main():
@@ -96,7 +98,15 @@ def test_read_nrao_loopback(tmp_path, nrao_uv):
 
     testfile = os.path.join(tmp_path, "ms_testfile.ms")
 
-    uvobj.write_ms(testfile)
+    with uvtest.check_warnings(
+        UserWarning,
+        match=[
+            "Writing in the MS file that the units of the data are",
+            "The uvw_array does not match the expected values",
+        ],
+    ):
+        uvobj.write_ms(testfile)
+
     uvobj2 = UVData()
     uvobj2.read_ms(testfile)
 
@@ -105,6 +115,9 @@ def test_read_nrao_loopback(tmp_path, nrao_uv):
     assert uvobj2.filename == ["ms_testfile.ms"]
     uvobj.filename = uvobj2.filename
 
+    # Test that the scan numbers are equal
+    assert (uvobj.scan_number_array == uvobj2.scan_number_array).all()
+
     assert uvobj == uvobj2
 
 
@@ -197,7 +210,8 @@ def test_read_ms_read_uvfits(nrao_uv, casa_uvfits):
     assert uvfits_uv != ms_uv
     uvfits_uv.integration_time = ms_uv.integration_time
     # they are equal if only required parameters are checked:
-    assert uvfits_uv.__eq__(ms_uv, check_extra=False)
+    # scan numbers only defined for the MS
+    assert uvfits_uv.__eq__(ms_uv, check_extra=False, allowed_failures=allowed_failures)
 
     # set those parameters to none to check that the rest of the objects match
     ms_uv.antenna_diameters = None
@@ -218,6 +232,9 @@ def test_read_ms_read_uvfits(nrao_uv, casa_uvfits):
     assert ms_uv.filename == ["day2_TDEM0003_10s_norx_1src_1spw.ms"]
     uvfits_uv.filename = ms_uv.filename
 
+    # propagate scan numbers to the uvfits, ONLY for comparison
+    uvfits_uv.scan_number_array = ms_uv.scan_number_array
+
     assert uvfits_uv == ms_uv
 
 
@@ -240,6 +257,9 @@ def test_read_ms_write_uvfits(nrao_uv, tmp_path):
     assert ms_uv.filename == ["day2_TDEM0003_10s_norx_1src_1spw.ms"]
     uvfits_uv.filename = ms_uv.filename
 
+    # propagate scan numbers to the uvfits, ONLY for comparison
+    uvfits_uv.scan_number_array = ms_uv.scan_number_array
+
     assert uvfits_uv == ms_uv
     del ms_uv
     del uvfits_uv
@@ -265,6 +285,9 @@ def test_read_ms_write_miriad(nrao_uv, tmp_path):
     assert ms_uv.filename == ["day2_TDEM0003_10s_norx_1src_1spw.ms"]
     miriad_uv.filename = ms_uv.filename
 
+    # propagate scan numbers to the miriad uvdata, ONLY for comparison
+    miriad_uv.scan_number_array = ms_uv.scan_number_array
+
     assert miriad_uv == ms_uv
 
 
@@ -277,6 +300,9 @@ def test_multi_files(casa_uvfits, axis):
     """
     uv_full = casa_uvfits.copy()
 
+    # Ensure the scan numbers are defined for the comparison
+    uv_full._set_scan_numbers()
+
     uv_multi = UVData()
     testfile1 = os.path.join(DATA_PATH, "multi_1.ms")
     testfile2 = os.path.join(DATA_PATH, "multi_2.ms")
@@ -323,7 +349,7 @@ def test_multi_files(casa_uvfits, axis):
     uv_multi.filename = uv_full.filename
     uv_multi._filename.form = (1,)
 
-    assert uv_multi == uv_full
+    assert uv_multi.__eq__(uv_full, allowed_failures=allowed_failures)
     del uv_full
     del uv_multi
 
@@ -387,6 +413,7 @@ def test_parse_pyuvdata_frame_ref_errors(check_warning, frame, epoch, msg):
         assert str(cm.value).startswith(msg)
 
 
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
 def test_ms_history_lesson(mir_uv, tmp_path):
     """
     Test that the MS reader/writer can parse complex history
@@ -429,6 +456,7 @@ def test_ms_history_lesson(mir_uv, tmp_path):
     assert ms_uv.history.startswith("  Read/written with pyuvdata version:")
 
 
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
 def test_ms_no_ref_dir_source(mir_uv, tmp_path):
     """
     Test that the MS writer/reader appropriately reads in a single-source data set
@@ -451,6 +479,7 @@ def test_ms_no_ref_dir_source(mir_uv, tmp_path):
     assert ms_uv.phase_center_catalog == mir_uv.phase_center_catalog
 
 
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
 def test_ms_multi_spw_data_variation(mir_uv, tmp_path):
     """
     Test that the MS writer/reader appropriately reads in a single-source data set
@@ -480,11 +509,16 @@ def test_ms_multi_spw_data_variation(mir_uv, tmp_path):
     assert np.all(ms_uv.integration_time == np.array([1.0]))
 
 
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
 @pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
-def test_ms_phasing(mir_uv, tmp_path):
+@pytest.mark.parametrize("future_shapes", [True, False])
+def test_ms_phasing(mir_uv, future_shapes, tmp_path):
     """
     Test that the MS writer can appropriately handle unphased data sets.
     """
+    if future_shapes:
+        mir_uv.use_future_array_shapes()
+
     ms_uv = UVData()
     testfile = os.path.join(tmp_path, "out_ms_phasing.ms")
 
@@ -504,10 +538,15 @@ def test_ms_phasing(mir_uv, tmp_path):
     )
 
 
-def test_ms_single_chan(mir_uv, tmp_path):
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
+@pytest.mark.parametrize("future_shapes", [True, False])
+def test_ms_single_chan(mir_uv, future_shapes, tmp_path):
     """
     Make sure that single channel writing/reading work as expected
     """
+    if future_shapes:
+        mir_uv.use_future_array_shapes()
+
     ms_uv = UVData()
     testfile = os.path.join(tmp_path, "out_ms_single_chan.ms")
 
@@ -534,6 +573,9 @@ def test_ms_single_chan(mir_uv, tmp_path):
     ms_uv.channel_width = np.array([ms_uv.channel_width])
     ms_uv.flex_spw_id_array = ms_uv.spw_array.copy()
 
+    if future_shapes:
+        ms_uv.use_future_array_shapes()
+
     # Finally, take care of the odds and ends
     ms_uv.extra_keywords = {}
     ms_uv.history = mir_uv.history
@@ -541,9 +583,80 @@ def test_ms_single_chan(mir_uv, tmp_path):
     ms_uv.instrument = mir_uv.instrument
     ms_uv.reorder_blts()
 
+    # propagate scan numbers to the uvfits, ONLY for comparison
+    mir_uv.scan_number_array = ms_uv.scan_number_array
+
     assert ms_uv == mir_uv
 
 
+@pytest.mark.filterwarnings("ignore:pamatten in extra_keywords is a list, array")
+@pytest.mark.filterwarnings("ignore:psys in extra_keywords is a list, array or dict")
+@pytest.mark.filterwarnings("ignore:psysattn in extra_keywords is a list, array or")
+@pytest.mark.filterwarnings("ignore:ambpsys in extra_keywords is a list, array or dict")
+@pytest.mark.filterwarnings("ignore:bfmask in extra_keywords is a list, array or dict")
+@pytest.mark.filterwarnings("ignore:LST values stored in this file are not ")
+@pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
+def test_ms_scannumber_multiphasecenter(tmp_path):
+    """
+    Make sure that single channel writing/reading work as expected
+    """
+    carma_file = os.path.join(DATA_PATH, "carma_miriad")
+    testfile = os.path.join(tmp_path, "carma_out.ms")
+
+    miriad_uv = UVData()
+
+    # Copied in from test_miriad.py::test_read_carma_miriad_write_ms
+    with uvtest.check_warnings(
+        UserWarning,
+        [
+            "Altitude is not present in Miriad file, "
+            "using known location values for SZA.",
+            "The uvw_array does not match the expected values given the antenna "
+            "positions.",
+            "pamatten in extra_keywords is a list, array or dict",
+            "psys in extra_keywords is a list, array or dict",
+            "psysattn in extra_keywords is a list, array or dict",
+            "ambpsys in extra_keywords is a list, array or dict",
+            "bfmask in extra_keywords is a list, array or dict",
+            "Cannot fix the phases of multi phase center datasets, as they were not "
+            "supported when the old phasing method was used, and thus, there "
+            "is no need to correct the data.",
+        ],
+    ):
+        miriad_uv.read(carma_file, fix_old_proj=True)
+
+    # MIRIAD is missing these in the file, so we'll fill it in here.
+    miriad_uv.antenna_diameters = np.zeros(miriad_uv.Nants_telescope)
+    miriad_uv.antenna_diameters[:6] = 10.0
+    miriad_uv.antenna_diameters[15:] = 3.5
+
+    # We need to recalculate app coords here for one source ("NOISE"), which was
+    # not actually correctly calculated in the online CARMA system (long story). Since
+    # the MS format requires recalculating apparent coords after read in, we'll
+    # calculate them here just to verify that everything matches.
+    miriad_uv._set_app_coords_helper()
+    miriad_uv.write_ms(testfile, clobber=True)
+
+    # Check on the scan number grouping based on consecutive integrations per phase
+    # center
+
+    # Double-check multi-phase center is True.
+    assert miriad_uv.multi_phase_center
+
+    # Read back in as MS. Should have 3 scan numbers defined.
+    ms_uv = UVData()
+    ms_uv.read(testfile)
+
+    assert np.unique(ms_uv.scan_number_array).size == 3
+    assert (np.unique(ms_uv.scan_number_array) == np.array([1, 2, 3])).all()
+
+    # The scan numbers should match the phase center IDs, offset by 1
+    # so that the scan numbers start with 1, not 0.
+    assert ((miriad_uv.phase_center_id_array == (ms_uv.scan_number_array - 1))).all()
+
+
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
 def test_ms_extra_data_descrip(mir_uv, tmp_path):
     """
     Make sure that data sets can be read even if the main table doesn't have data
@@ -583,10 +696,14 @@ def test_ms_extra_data_descrip(mir_uv, tmp_path):
     mir_uv.extra_keywords = ms_uv.extra_keywords
     mir_uv.filename = ms_uv.filename = None
 
+    # propagate scan numbers to the miriad uvdata, ONLY for comparison
+    mir_uv.scan_number_array = ms_uv.scan_number_array
+
     # Finally, with all exceptions handled, check for equality.
     assert ms_uv == mir_uv
 
 
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
 def test_ms_weights(mir_uv, tmp_path):
     """
     Test that the MS writer/reader appropriately handles data when the
@@ -616,6 +733,7 @@ def test_ms_weights(mir_uv, tmp_path):
     assert np.all(ms_uv.nsample_array == 1.0)
 
 
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
 @pytest.mark.parametrize(
     "badcol,badval,errtype,msg",
     (
@@ -662,3 +780,23 @@ def test_ms_reader_errs(mir_uv, tmp_path, badcol, badval, errtype, msg):
     with pytest.raises(errtype) as cm:
         ms_uv.read_ms(testfile, data_column=data_col)
     assert str(cm.value).startswith(msg)
+
+
+def test_antenna_diameter_handling(hera_uvh5, tmp_path):
+    uv_obj = hera_uvh5
+
+    uv_obj.antenna_diameters = np.asarray(uv_obj.antenna_diameters, dtype=">f4")
+
+    test_file = os.path.join(tmp_path, "dish_diameter_out.ms")
+    with uvtest.check_warnings(
+        UserWarning, match="Writing in the MS file that the units of the data are"
+    ):
+        uv_obj.write_ms(test_file, force_phase=True)
+
+    uv_obj2 = UVData.from_file(test_file)
+
+    # MS write/read adds some stuff to history & extra keywords
+    uv_obj2.history = uv_obj.history
+    uv_obj2.extra_keywords = uv_obj.extra_keywords
+
+    assert uv_obj2.__eq__(uv_obj, allowed_failures=allowed_failures)
